﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class Enemy : MonoBehaviour
{
	[SerializeField]
	private int maxHealth;
	[SerializeField]
	private int curHealth;

	private MeshRenderer meshRenderer;

	[SerializeField]
	private ParticleSystem bloodEffect;

	private RectTransform healthUIRect;
	private Slider healthUI;

	private Collider col;

	private void Start()
	{
		meshRenderer = GetComponent<MeshRenderer>();
		col = GetComponent<Collider>();
		curHealth = maxHealth;
	}

	private void Update()
	{
		UpdateHealthUIRect();
	}

	public void GetDamaged(int dmg)
	{
		curHealth -= dmg;
		StartCoroutine(DamageRoutine());
		UpdateHealthUI();

		if (curHealth <= 0)
		{
			Dead();
		}
	}

	private void Dead()
	{
		Debug.Log(gameObject + " : DEAD!");
		GetComponent<Rigidbody>().useGravity = false;
		GetComponent<Collider>().enabled = false;
		if (healthUI != null)
			Destroy(healthUI.gameObject);
		Destroy(gameObject, 5);
	}

	private IEnumerator DamageRoutine()
	{
		bloodEffect?.Play();
		yield return null;
	}

	private void UpdateHealthUI()
	{
		if (healthUI == null)
		{
			healthUI = IngameUIManager.inst.CreateNewHealthUI();
			healthUIRect = healthUI.GetComponent<RectTransform>();
		}
		IngameUIManager.inst.UpdateEnemyHealthUI(healthUI, maxHealth, curHealth);
	}

	private void UpdateHealthUIRect()
	{
		if (healthUI == null)
			return;

		float distance = Vector3.Distance(Camera.main.transform.position, transform.position);
		healthUIRect.localScale = new Vector3(1,1,1) / distance;

		Vector2 viewportPoint = Camera.main.WorldToViewportPoint(transform.position + new Vector3(0,col.bounds.extents.y * 1.5f));
		healthUIRect.anchorMax = viewportPoint;
		healthUIRect.anchorMin = viewportPoint;
	}
}
